#!/usr/bin/python

import sys

def rotate(l,n):
        return l[n:] + l[:n]

class UcodeC:
    def __init__(self):
        pass

    def parse(self, ucode_file):
        ucode = open(ucode_file, 'r')
        params = {'data_bits': None,
                  'addr_bits': None,
                  'opcode_ticks': None,
                  'rotate_ticks': 0,
                  'comments': 'no'}
        signals = {}
        ticks = {}
        current_line = 0

        print "Parsing ucode file..."
        for line in ucode:
            current_line += 1
            tokens = line.split()
            # discard empty lines and comments
            if len(line) == 0 or len(tokens) == 0:
                continue
            if line[0] == '#':
                continue

            # handle param assignements
            if tokens[0] in params:
                if len(tokens) < 2:
                    print "invalid param assignement at line %d" % current_line
                    continue
                params[tokens[0]] = tokens[1]
                continue
            
            # handle signal declaration
            if tokens[0] == 'signal':
                if len(tokens) < 4:
                    print "invalid signal declaration at line %d" % current_line
                    continue
                signals[tokens[1]] = {'pin_no': int(tokens[2]), 'default_value': tokens[3]}
                continue

            # handle opcode token
            if tokens[0] == 'opcode':
                if len(tokens) < 2:
                    print "invalid opcode declaration at line %d" % current_line
                    continue
                current_opcode = int(tokens[1])
                if len(tokens) >= 3:
                    current_opcode_name = ' '.join(tokens[2:])
                else:
                    current_opcode_name = ""
                ticks[current_opcode] = []
                continue

            # handle tick declaration
            if tokens[0] == 'tick':
                tick = {'__comment': ' '.join(tokens[1:]),
                        '__opcode_name': current_opcode_name}
                for assign in tokens[1:]:
                    parts = assign.split('=')
                    if len(parts) != 2:
                        print ("malformed assignement '%s' inside a tick at line %d"
                        % (assign, current_line))
                        continue
                    tick[parts[0]] = parts[1]
                ticks[current_opcode].append(tick)
                continue

            print "unrecognized token '%s' at line %d" % (tokens[0], current_line)
        print "Done."
        self.params = params
        self.signals = signals
        self.ticks = ticks
    
    # check required params
    def check_params(self):
        for param in self.params:
            if self.params[param] is None:
                print "required param '%s' was not set" % param
                return False
        return True

    def output(self, dat_file):
        out_stream = open(dat_file, 'w')
        params = self.params
        ticks = self.ticks
        signals = self.signals

        # generate output
        print "Writing microcode ROM..."
        num_ticks = 2**int(params['addr_bits'])
        data_bits = int(params['data_bits'])
        opcode_ticks = int(params['opcode_ticks'])
        num_opcodes = num_ticks/opcode_ticks
        print "ticks : %d" % num_ticks
        print "ticks per opcode: %d" % opcode_ticks
        print "number of opcodes: %d" % num_opcodes
        print "number of control signals: %d" % data_bits

        default_bits = ['0'] * data_bits
        for name in signals:
            signal = signals[name]
            default_bits[signal['pin_no']] = signal['default_value']
        print "default control state: %s" % ''.join(reversed(default_bits))

        for opcode_n in range(num_opcodes):
            opcode_lines = []
            bits = list(default_bits)
            for tick_n in range(opcode_ticks):
                comment = ''
                if opcode_n in ticks and tick_n < len(ticks[opcode_n]):
                    tick = ticks[opcode_n][tick_n]
                    if params['comments'] != 'no':
                        comment = " -- %s(%d), tick %d" % (tick['__opcode_name'], opcode_n, tick_n)
                    if params['comments'] == 'verbose':
                        comment += ", %s" % tick['__comment']
                    for signal_name in tick:
                        # skip tick annotations
                        if signal_name[0:2] == '__':
                            continue
                        signal = signals[signal_name]
                        bits[signal['pin_no']] = tick[signal_name]
                opcode_lines.append("%s%s\n" % (''.join(reversed(bits)), comment))
            for line in rotate(opcode_lines,int(params['rotate_ticks'])):
                out_stream.write(line)
        
if __name__ == '__main__':
    if len(sys.argv) != 3:
        print "usage: %s file.ucode file.dat" % sys.argv[0]
        sys.exit(1)

    comp = UcodeC()
    comp.parse(sys.argv[1])
    comp.check_params()
    comp.output(sys.argv[2])
