library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use std.textio.all;

use work.txt_util.all;

entity ram is

    generic (
        DATA_BITS : natural;
        ADDR_BITS : natural;
        DATA_FILE : string;
        TIME_RC : time;
        TIME_HZ : time
    ); 

    port (
         addr    : in std_logic_vector (ADDR_BITS-1 downto 0);
         wr_n	 : in std_logic;
         oe		 : in std_logic;
         data    : inout std_logic_vector (DATA_BITS-1 downto 0)
     );

end ram;

architecture rtl of ram is

    type data_t is array(0 to 2**ADDR_BITS-1) of bit_vector(DATA_BITS-1 downto 0);

    impure function init_data (data_file_name : in string) return data_t is
        file datafile : text is in data_file_name;
        variable line_read : line;
        variable data_to_return : data_t;
    begin
        for i in data_t'range loop
            readline(datafile, line_read);
            read(line_read, data_to_return(i));
        end loop;
        return data_to_return;
    end function;
    
    signal contents : data_t := init_data(DATA_FILE);

begin
    data <= to_stdlogicvector(contents(to_integer(unsigned(addr))))
        after TIME_RC
        when (wr_n = '1' and oe = '1')
        else (others => 'Z')
        after TIME_HZ;

    process(wr_n)
    begin
        -- this is dirty, but at start undefined values get written otherwise !
        if wr_n'event and wr_n = '1' and (data(0) = '1' or data(0) = '0') then
            contents(to_integer(unsigned(addr))) <= to_bitvector(data);
            report "written " & hstr(data) & " at addr " & hstr(addr);
        end if;
    end process;
    
end rtl;
