library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
 
entity ram_tb is
end entity;
 
architecture TB of ram_tb is

	component ram

        generic (
            DATA_BITS : natural;
            ADDR_BITS : natural;
            DATA_FILE : string;
            TIME_RC : time;
            TIME_HZ : time
        ); 

        port (
             addr    : in std_logic_vector (ADDR_BITS-1 downto 0);
             wr_n	 : in std_logic;
             oe		 : in std_logic;
             data    : inout std_logic_vector (DATA_BITS-1 downto 0)
         );

	end component;

	signal addr     : std_logic_vector (3 downto 0) := X"0";
	signal wr_n		: std_logic := '1';
	signal oe		: std_logic := '1';
	signal data     : std_logic_vector (15 downto 0) := X"AAAA";
 
begin

	r1 : ram
    generic map (
		DATA_BITS => 16,
		ADDR_BITS => 4,
		DATA_FILE => "test/ram.dat",
        TIME_RC => 15 ns,
        TIME_HZ => 8 ns
	)
	port map (
        addr => addr,
        wr_n => wr_n,
        oe => oe,
        data => data
    );
    
    process
    begin
        data <= (others => 'Z');
        -- test read
        -- read at 0x03
		addr <= X"3";
		wait for 100 ns;
        -- read at 0x07
		addr <= X"7";
		wait for 100 ns;
        
        -- test write
        -- write byte at 0x0A
        -- set addr and request write
        addr <= X"A";
        wr_n <= '0';
        -- wait for chip to release data lines
		wait for 10 ns;
        -- write data and wait for it to settle
        data <= X"4242";
		wait for 10 ns;
        wr_n <= '1';
        -- validate write
		wait for 10 ns;
        -- release the line
        data <= (others => 'Z');
		wait for 100 ns;
        -- read back
        addr <= X"A";
		wait for 100 ns;
    end process;

end;
